package info.textgrid.middleware;

import java.io.IOException;
import java.text.ParseException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.action.search.SearchScrollRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.RangeQueryBuilder;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.json.JSONObject;
import info.textgrid.middleware.oaipmh.ListIdentifiersType;
import info.textgrid.middleware.oaipmh.ResumptionTokenType;

/**
 *
 */
public class IdentifierListDelivererIDIOM extends IdentifierListDelivererAbstract {

  private static Log log = LogFactory.getLog(IdentifierListDelivererIDIOM.class);

  /**
   * @param textgrid
   * @param dariah
   */
  public IdentifierListDelivererIDIOM(boolean textgrid, boolean dariah) {
    super(textgrid, dariah);
  }

  /*
   * (non-Javadoc)
   * 
   * @see info.textgrid.middleware.IdentifierListDelivererAbstract#processIdentifierList(java.lang.
   * String, java.lang.String, java.lang.String, java.lang.String)
   */
  public ListIdentifiersType processIdentifierList(final String from, final String to,
      final String set, final String resumptionToken) throws ParseException, IOException {

    // String resTokenValue = resumptionToken;

    ListIdentifiersType identifierList = new ListIdentifiersType();

    BoolQueryBuilder recordFilterForClassicMayan;
    RangeQueryBuilder rangeQuery;

    rangeQuery = QueryBuilders.rangeQuery("lastModified").from(from).to(to);

    recordFilterForClassicMayan = QueryBuilders.boolQuery().must(rangeQuery)
        .must(QueryBuilders.matchPhraseQuery("project.id",
            "TGPR-0e926f53-1aba-d415-ecf6-539edcd8a318"))
        .must(QueryBuilders.matchPhraseQuery("format", "text/tg.inputform+rdf+xml"))
        .must(QueryBuilders.matchPhraseQuery("notes", "ARTEFACT"));


    // List<String> artefactURIs = new ArrayList<String>();
    // Queries queries = new Queries();

    SearchResponse scrollResp;

    SearchRequest searchRequest = new SearchRequest("textgrid-nonpublic");
    SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();

    scrollResp = null;

    searchSourceBuilder.query(recordFilterForClassicMayan);
    searchSourceBuilder.size(30);
    searchRequest.source(searchSourceBuilder);

    if (resumptionToken != null) {
      SearchScrollRequest scrollRequest = new SearchScrollRequest(resumptionToken);
      scrollRequest.scroll(TimeValue.timeValueHours(24L));
      try {
        scrollResp = OAI_ESClient.getEsClient().scroll(scrollRequest, RequestOptions.DEFAULT);
      } catch (IOException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    } else {
      // Use given resumption token here.
      searchRequest.source(searchSourceBuilder);
      searchRequest.scroll(TimeValue.timeValueHours(24L));
      try {
        scrollResp = OAI_ESClient.getEsClient().search(searchRequest, RequestOptions.DEFAULT);
      } catch (IOException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
      // scrollResp = OAI_ESClient.getEsClient().prepareSearchScroll(resumptionToken)
      // .setScroll(TimeValue.timeValueHours(24L)).execute().actionGet();
    }

    String scrollID = scrollResp.getScrollId();

    long completeListSize = scrollResp.getHits().totalHits;

    // long listSize = Queries.getAmountOfArtefacts();
    if (completeListSize > 0) {
      setFoundItems(true);
      int i = 0;

      for (SearchHit hit : scrollResp.getHits().getHits()) {
        i++;
        String textgridURI =
            OAIPMHUtilities.fieldLoader(new JSONObject(hit.getSourceAsMap()), "textgridUri");
        // hit.getFields().get("textgridUri").getValue().toString().replace(".0", "");

        log.debug("TewxtGrid URI: " + textgridURI);

        RecordDelivererIDIOM idiomRecord = new RecordDelivererIDIOM(true, false);

        identifierList.getHeader()
            .add(idiomRecord.getRecordById(textgridURI.replace(".0", "")).getRecord().getHeader());
      }
      /*
       * for (String artefactURI : artefactURIs) { RecordDelivererIDIOM idiomRecord = new
       * RecordDelivererIDIOM(true, false); identifierList.getHeader()
       * .add(idiomRecord.getRecordById(artefactURI).getRecord().getHeader()); i++; }
       */

      // Check the need for a resumption token!
      ResumptionTokenType resTokenForResponse = OAIPMHUtilities.getResumptionToken(completeListSize,
          resumptionToken, cursorCollector, scrollID, 30, i);
      if (resTokenForResponse != null) {
        identifierList.setResumptionToken(resTokenForResponse);
      }
    } else {
      setFoundItems(false);
    }
    return identifierList;
  }

}
