package info.textgrid.middleware;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.util.Properties;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.datatype.DatatypeConfigurationException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.elasticsearch.common.Strings;
import org.json.JSONException;
import org.json.JSONObject;
import org.springframework.stereotype.Component;
import info.textgrid.middleware.oaipmh.ContributorType;
import info.textgrid.middleware.oaipmh.DateType;
import info.textgrid.middleware.oaipmh.GetRecordType;
import info.textgrid.middleware.oaipmh.HeaderType;
import info.textgrid.middleware.oaipmh.OAIPMHtype;
import info.textgrid.middleware.oaipmh.ObjectFactory;
import info.textgrid.middleware.oaipmh.OpenaireDescriptionType;
import info.textgrid.middleware.oaipmh.RelatedIdentifierType;
import info.textgrid.middleware.oaipmh.RelationType;
import info.textgrid.middleware.oaipmh.Resource;
import info.textgrid.middleware.oaipmh.Resource.AlternateIdentifiers;
import info.textgrid.middleware.oaipmh.Resource.AlternateIdentifiers.AlternateIdentifier;
import info.textgrid.middleware.oaipmh.Resource.Contributors;
import info.textgrid.middleware.oaipmh.Resource.Contributors.Contributor;
// import info.textgrid.middleware.oaipmh.Resource.Contributors.Contributor.NameIdentifier;
import info.textgrid.middleware.oaipmh.Resource.Creators;
import info.textgrid.middleware.oaipmh.Resource.Creators.Creator;
import info.textgrid.middleware.oaipmh.Resource.Creators.Creator.NameIdentifier;
import info.textgrid.middleware.oaipmh.Resource.Dates;
import info.textgrid.middleware.oaipmh.Resource.Dates.Date;
import info.textgrid.middleware.oaipmh.Resource.Descriptions;
import info.textgrid.middleware.oaipmh.Resource.Descriptions.Description;
import info.textgrid.middleware.oaipmh.Resource.Formats;
import info.textgrid.middleware.oaipmh.Resource.GeoLocations;
import info.textgrid.middleware.oaipmh.Resource.GeoLocations.GeoLocation;
import info.textgrid.middleware.oaipmh.Resource.Identifier;
import info.textgrid.middleware.oaipmh.Resource.RelatedIdentifiers;
import info.textgrid.middleware.oaipmh.Resource.RelatedIdentifiers.RelatedIdentifier;
import info.textgrid.middleware.oaipmh.Resource.ResourceType;
import info.textgrid.middleware.oaipmh.Resource.RightsList;
import info.textgrid.middleware.oaipmh.Resource.RightsList.Rights;
import info.textgrid.middleware.oaipmh.Resource.Sizes;
import info.textgrid.middleware.oaipmh.Resource.Subjects;
import info.textgrid.middleware.oaipmh.Resource.Subjects.Subject;
import info.textgrid.middleware.oaipmh.Resource.Titles;
import info.textgrid.middleware.oaipmh.Resource.Titles.Title;
import info.textgrid.middleware.oaipmh.TitleType;
// import info.textgrid.middleware.oaipmh.ResourceType;

/**
 * 
 * @author Maximilian Brodhun, SUB
 * @version 1.0.0
 * @since 13.06.2020
 *
 */

@Component
public class OpenAireRecord extends RecordDelivererAbstract {

  // Logger for errors and info
  private static Log log = LogFactory.getLog(OpenAireRecord.class);

  // Array containing all fields to get from the edition dataset in elasticsearch
  private String[] fields;

  // Container object for the result of the elasticsearch request. This object contains the source
  // of the response
  private JSONObject jsonObj = new JSONObject();

  // This variables contains the field requested in the ElasticSearch Index. The content will be
  // given by a configuration file
  private String[] titleFields;
  private String[] dateFields;
  private String[] contributorFields;
  private String[] languageFields;
  private String[] alternateIdentifierFields;
  private String[] formatFields;
  private String[] rightsFields;
  private String[] descriptionFields;
  private String[] relatedIdentifierFields;
  private String[] workFields;
  private String[] geoLocationFields;
  private String[] versionFields;
  private String[] subjectFields;
  private String relationToWorkObject;
  private String identifierField;
  private String formatField;
  private String handle;
  private String sizeField;

  // Contains the setting to get connection to the ElasticSearch index. The content will be given by
  // a configuration file
  private OAI_ESClient oaiEsClient;

  // This variable contains the XML-content for a whole OpenAire file
  private Resource resource = new Resource();

  /**
   * 
   * @param textgrid flag to indicate that the content of the record is context of the TextGrid repo
   * @param dariah flag to indicate that the content of the record is context of the DARIAH repo
   * @throws IOException
   */
  public OpenAireRecord(boolean textgrid, boolean dariah) throws IOException {
    super(textgrid, dariah);

    // Read the fields to request from elasticsearch from content file
    InputStream input = null;
    try {
      input = new FileInputStream("/etc/dhrep/oaipmh/oaipmh.properties");
    } catch (FileNotFoundException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    Properties prop = new Properties();
    prop.load(input);
    this.setHandle(prop.getProperty("oar.handle"));
    this.setFields(OAIPMHUtilities.fetchArrayFromPropFile("fields", prop)[0]);
    this.setFormatField(prop.getProperty("formatField"));
    this.setTitleFields(OAIPMHUtilities.fetchArrayFromPropFile("oar.titleFields", prop)[0]);
    this.setDateFields(OAIPMHUtilities.fetchArrayFromPropFile("oar.dateFields", prop)[0]);
    this.setContributorFields(
        OAIPMHUtilities.fetchArrayFromPropFile("oar.contributorFields", prop)[0]);
    this.setLanguageFields(OAIPMHUtilities.fetchArrayFromPropFile("oar.languageFields", prop)[0]);
    this.setAlternateIdentifierFields(
        OAIPMHUtilities.fetchArrayFromPropFile("oar.alternateIdentifierFields", prop)[0]);
    this.setFormatFields(OAIPMHUtilities.fetchArrayFromPropFile("oar.formatFields", prop)[0]);
    this.setRightsFields(OAIPMHUtilities.fetchArrayFromPropFile("oar.rightsFields", prop)[0]);
    this.setDescriptionFields(
        OAIPMHUtilities.fetchArrayFromPropFile("oar.descriptionFields", prop)[0]);
    this.setRelationToWorkObject(prop.getProperty("oar.relationToWorkObject"));
    this.setRelatedIdentifierFields(
        OAIPMHUtilities.fetchArrayFromPropFile("oar.relatedIdentifierFields", prop)[0]);
    this.setGeoLocationFields(
        OAIPMHUtilities.fetchArrayFromPropFile("oar.geoLocationFields", prop)[0]);
    this.setVersionFields(OAIPMHUtilities.fetchArrayFromPropFile("oar.versionFields", prop)[0]);
    this.setSubjectFields(OAIPMHUtilities.fetchArrayFromPropFile("oar.subjectFields", prop)[0]);
    this.setSizeField(prop.getProperty("oar.sizeField"));
  }

  /**
   * building the record XML object for the OAI-PMH response
   * 
   * @throws ParseException
   * @throws IOException
   */
  public GetRecordType getRecordById(String idInElasticSearchIndex)
      throws ParseException, DatatypeConfigurationException, IOException {
    this.jsonObj = new JSONObject(OAIPMHUtilities
        .getRcordByIDFromElasticSearch(OAI_ESClient.getEsIndex(),
            idInElasticSearchIndex.replace("textgrid:", ""), this.fields,
            Strings.EMPTY_ARRAY)
        .getSource());

    HeaderType header = new HeaderType();
    header.setDatestamp(OAIPMHUtilities
        .datestampAsString(OAIPMHUtilities.fieldLoader(this.jsonObj, TGConstants.CREATED)));

    if (!idInElasticSearchIndex.startsWith(TGConstants.ITEM_IDENTIFIER_PREFIX)) {
      header.setIdentifier(TGConstants.ITEM_IDENTIFIER_PREFIX + idInElasticSearchIndex);
    } else {
      header.setIdentifier(idInElasticSearchIndex);
    }

    GetRecordType grt = OAIPMHUtilities.getRecordType(this.setOpenAireRecord(), header);

    return grt;
  }

  /**
   * Put the record from open-aire into the JAXBElement container to add it to an oai-pmh object
   * 
   * @param idInElasticSearchIndex
   * @return
   * @throws ParseException
   * @throws JAXBException
   * @throws DatatypeConfigurationException
   * @throws IOException
   */
  public JAXBElement<OAIPMHtype> getRecord(String idInElasticSearchIndex)
      throws ParseException, JAXBException, DatatypeConfigurationException, IOException {
    ObjectFactory obf = new ObjectFactory();
    JAXBElement<OAIPMHtype> oaijaxb;
    OAIPMHtype oai = new OAIPMHtype();
    HeaderType header = new HeaderType();

    this.jsonObj = new JSONObject(OAIPMHUtilities
        .getRcordByIDFromElasticSearch(OAI_ESClient.getEsIndex(), idInElasticSearchIndex,
            this.fields, Strings.EMPTY_ARRAY)
        .getSource());

    // Setting the header for the oaipmh response
    header.setDatestamp(OAIPMHUtilities
        .datestampAsString(OAIPMHUtilities.fieldLoader(this.jsonObj, TGConstants.CREATED)));
    // The id for the response has to be starts with "textgrid:". The id in elasticsearch is
    // missing. In consequence is optional in oaipmh
    // request to write textgrid:1233.0 or jsut 1233.0. For the response the prefix has to be added
    if (!idInElasticSearchIndex.startsWith(TGConstants.ITEM_IDENTIFIER_PREFIX)) {
      header.setIdentifier(TGConstants.ITEM_IDENTIFIER_PREFIX + idInElasticSearchIndex);
    } else {
      header.setIdentifier(idInElasticSearchIndex);
    }
    // It is necessary to add the GetRecordType to the OAIPMH object.
    GetRecordType grt = OAIPMHUtilities.getRecordType(this.setOpenAireRecord(), header);
    oai.setGetRecord(grt);
    oaijaxb = obf.createOAIPMH(oai);
    oaijaxb.setValue(oai);

    return oaijaxb;
  }

  /**
   * Adding the open aire record fields to the openaire record object
   * 
   * @param idInElasticSearchIndex
   * @return
   * @throws ParseException
   * @throws DatatypeConfigurationException
   * @throws IOException
   */
  private Resource setOpenAireRecord()
      throws ParseException, DatatypeConfigurationException, IOException {

    this.resource.setTitles(this.addTitles());
    this.resource.setPublisher(this.addPublisher());
    this.resource.setPublicationYear(this.addPublicationYear());
    this.resource.setDates(this.addDates());
    this.resource.setContributors(this.addContributors());
    this.resource.setLanguage(this.addLanguae());
    this.resource.setAlternateIdentifiers(this.addAlternateIdentifier());
    this.resource.setFormats(this.addFormats());
    this.resource.setRightsList(this.addRights());
    this.resource.setDescriptions(this.addDescriptions());
    this.resource.setRelatedIdentifiers(this.addRelatedIdentifiers());
    this.resource.setIdentifier(this.addIdentifier());
    this.resource.setCreators(this.addCreators());
    this.addGeoLocation();
    this.resource.setResourceType(this.addResourceType());
    this.resource.setVersion(this.addVersion());

    if (addSubjects().getSubject().get(0).getValue() != null) {
      this.resource.setSubjects(this.addSubjects()); // TODO what if subjects == null?
    }

    this.resource.setSizes(this.addSize());


    return this.resource;
  }

  private Sizes addSize() {
    Sizes sizes = new Sizes();

    sizes.getSize().add(OAIPMHUtilities.fieldLoader(this.jsonObj, this.sizeField) + " Bytes");
    return sizes;
  }


  private Subjects addSubjects() {

    Subjects subjects = new Subjects();

    for (String subjectField : this.subjectFields) {
      Subject subject = new Subject();
      subject.setValue(OAIPMHUtilities.fieldLoader(this.jsonObj, subjectField + ".value"));
      subject.setSchemeURI(OAIPMHUtilities.fieldLoader(this.jsonObj, subjectField + ".id.type"));
      subject
          .setSubjectScheme(OAIPMHUtilities.fieldLoader(this.jsonObj, subjectField + ".id.value"));
      subjects.getSubject().add(subject);
    }
    if (subjects.getSubject().get(0) != null) {
      return subjects;
    } else {
      return null;
    }

  }

  /**
   * @return
   */
  private String addVersion() {
    String version = "";
    for (String versionField : this.versionFields) {
      version = OAIPMHUtilities.fieldLoader(this.jsonObj, versionField);
    }

    return version;
  }

  /**
   * @return
   */
  private static ResourceType addResourceType() {
    ResourceType resourceType = new ResourceType();

    resourceType.setResourceTypeGeneral(info.textgrid.middleware.oaipmh.ResourceType.DATASET);

    return resourceType;
  }

  /**
   * 
   */
  private void addGeoLocation() {
    GeoLocations geoLocations = new GeoLocations();
    for (String geoLocationField : this.geoLocationFields) {
      GeoLocation geoLocation = new GeoLocation();
      geoLocation
          .setGeoLocationPlace(
              OAIPMHUtilities.fieldLoader(this.jsonObj, geoLocationField + ".value"));
      geoLocations.getGeoLocation().add(geoLocation);
    }
    if (geoLocations.getGeoLocation().get(0).getGeoLocationPlace() != null) {
      this.resource.setGeoLocations(geoLocations);
    }
    // return geoLocations;
  }

  /**
   * @return
   * @throws IOException
   */
  private RelatedIdentifiers addRelatedIdentifiers() throws IOException {
    RelatedIdentifiers relatedIdentifiers = new RelatedIdentifiers();

    for (String relatedIdentifierField : this.relatedIdentifierFields) {
      String[] workFields = {this.handle};
      RelatedIdentifier relatedIdentifier = new RelatedIdentifier();
      relatedIdentifier.setRelatedIdentifierType(RelatedIdentifierType.HANDLE);
      // relatedIdentifier.setRelatedMetadataScheme(TGConstants.TEXTGRID_METADATASCHEME);
      relatedIdentifier.setRelationType(RelationType.IS_PART_OF);
      // relatedIdentifier.setSchemeType("XSD");
      // relatedIdentifier.setSchemeURI(TGConstants.TEXTGRID_METADATASCHEME_URI);

      String uriForWork =
          OAIPMHUtilities.fieldLoader(this.jsonObj, relatedIdentifierField).replace("textgrid:",
              "");
      relatedIdentifier.setValue(OAIPMHUtilities.fieldLoader(new JSONObject(OAIPMHUtilities
          .getRcordByIDFromElasticSearch(OAI_ESClient.getEsIndex(), uriForWork, workFields,
              Strings.EMPTY_ARRAY)
          .getSource()),
          this.handle));
      relatedIdentifiers.getRelatedIdentifier().add(relatedIdentifier);
    }

    return relatedIdentifiers;
  }

  /**
   * @return
   * @throws IOException
   */
  private Descriptions addDescriptions() throws IOException {
    Descriptions descriptions = new Descriptions();

    for (String descriptionField : this.descriptionFields) {
      Description description = new Description();
      // description.setOpenaireDescriptionType(OpenaireDescriptionType.ABSTRACT);
      // description.setOpenaireDescriptionType(OpenaireDescriptionType.ABSTRACT);
      description.setDescriptionType(OpenaireDescriptionType.ABSTRACT);
      String idForWorkObject =
          OAIPMHUtilities.fieldLoader(this.jsonObj, this.relationToWorkObject).replace("textgrid:",
              "");
      JSONObject resultOfFurtherObject = new JSONObject(OAIPMHUtilities
          .getRcordByIDFromElasticSearch(OAI_ESClient.getEsIndex(), idForWorkObject,
              this.workFields, Strings.EMPTY_ARRAY)
          .getSource());
      String abstractFoFWork =
          OAIPMHUtilities.fieldLoader(
              resultOfFurtherObject,
              descriptionField);
      description.setLang(abstractFoFWork);
      descriptions.getDescription().add(description);
    }
    return descriptions;
  }

  /**
   * @return
   */
  private RightsList addRights() {
    RightsList rightsList = new RightsList();
    for (String rightsField : this.rightsFields) {
      Rights rights = new Rights();
      rights.setRightsURI(OAIPMHUtilities.fieldLoader(this.jsonObj, rightsField + ".licenseUri"));
      rights.setValue(OAIPMHUtilities.fieldLoader(this.jsonObj, rightsField + ".value"));
      rightsList.getRights().add(rights);
    }
    if (rightsList.getRights().get(0).getValue() != null) {
      Rights openAccesRight = new Rights();
      openAccesRight.setRightsURI("info:eu-repo/semantics/openAccess");
      rightsList.getRights().add(openAccesRight);
    }

    return rightsList;
  }

  /**
   * @return
   */
  private Formats addFormats() {
    Formats formats = new Formats();
    for (String formatField : this.formatFields) {
      formats.getFormat().add(OAIPMHUtilities.fieldLoader(this.jsonObj, formatField));
    }
    return formats;
  }

  /**
   * @return
   */
  private AlternateIdentifiers addAlternateIdentifier() {

    AlternateIdentifiers alternateIdentifiers = new AlternateIdentifiers();
    for (String alternateIdentifierField : this.alternateIdentifierFields) {
      AlternateIdentifier alternateIdentifier = new AlternateIdentifier();
      alternateIdentifier
          .setValue(OAIPMHUtilities.fieldLoader(this.jsonObj, alternateIdentifierField));
      alternateIdentifier.setAlternateIdentifierType(TGConstants.URI_IDENTIIER_TYPE);
      alternateIdentifiers.getAlternateIdentifier().add(alternateIdentifier);
    }

    return alternateIdentifiers;
  }

  /**
   * @return
   */
  private String addLanguae() {

    String language = null;
    for (String languageField : this.languageFields) {
      language = OAIPMHUtilities.fieldLoader(this.jsonObj, languageField);
    }

    return language;
  }

  /**
   * @return
   */
  private Contributors addContributors() {
    Contributors contributors = new Contributors();
    for (String contributorField : this.contributorFields) {
      Contributor contributorInOpenAireRecord = new Contributor();
      if (contributorField.equals("project")) {
        contributorInOpenAireRecord
            .setContributorName(
                OAIPMHUtilities.fieldLoader(this.jsonObj, contributorField + ".value"));
        contributorInOpenAireRecord.setContributorType(ContributorType.OTHER);
        info.textgrid.middleware.oaipmh.Resource.Contributors.Contributor.NameIdentifier name =
            new info.textgrid.middleware.oaipmh.Resource.Contributors.Contributor.NameIdentifier();
        name.setNameIdentifierScheme("textgrid");
        name.setValue(OAIPMHUtilities.fieldLoader(this.jsonObj, contributorField + ".id"));
        name.setSchemeURI("http://www.textgridlab.org/schema/textgrid-metadata_2010.xsd");
        contributorInOpenAireRecord.setNameIdentifier(name);

      } else {
        contributorInOpenAireRecord
            .setContributorName(OAIPMHUtilities.fieldLoader(this.jsonObj, contributorField));
        contributorInOpenAireRecord.setContributorType(ContributorType.DATA_MANAGER);
        /*
         * info.textgrid.middleware.oaipmh.Resource.Contributors.Contributor.NameIdentifier name =
         * new info.textgrid.middleware.oaipmh.Resource.Contributors.Contributor.NameIdentifier();
         * name.setNameIdentifierScheme("textgrid"); name.setValue("");
         * name.setSchemeURI("http://www.textgridlab.org/schema/textgrid-metadata_2010.xsd");
         * contributorInOpenAireRecord.setNameIdentifier(name);
         */
      }
      contributors.getContributor().add(contributorInOpenAireRecord);
    }

    return contributors;
  }

  /**
   * @return
   */
  private Dates addDates() {

    Dates dates = new Dates();

    for (String dateField : this.dateFields) {
      Date dateInOpenAireRecord = new Date();

      log.debug("date field: " + dateField);

      try {
        dateInOpenAireRecord.setValue(
            OAIPMHUtilities
                .datestampAsString(OAIPMHUtilities.fieldLoader(this.jsonObj, dateField)));
      } catch (ParseException e) {
        // TODO Auto-generated catch block
        log.error("Could not parse date field");
      }
      if (dateField.equals("created")) {
        dateInOpenAireRecord.setDateType(DateType.CREATED);
      }
      if (dateField.equals("issued")) {
        dateInOpenAireRecord.setDateType(DateType.ISSUED);
      }
      if (dateField.equals("lastModified")) {
        dateInOpenAireRecord.setDateType(DateType.UPDATED);
      }
      dates.getDate().add(dateInOpenAireRecord);
    }

    return dates;
  }

  /**
   * @return
   */
  private Identifier addIdentifier() {
    Identifier identifier = new Identifier();
    identifier.setIdentifierType("Handle");

    log.debug("TextGrid URI: " + this.jsonObj.get("textgridUri").toString());

    try {
      identifier.setValue(this.jsonObj.getJSONObject("pid").get("value").toString());

    } catch (JSONException fieldNotFound) {
      identifier.setValue(this.jsonObj.get("textgridUri").toString());
    }

    return identifier;
  }

  /**
   * @return
   */
  private Creators addCreators() {

    Creators creators = new Creators();
    /**
     * Creators Field
     */
    // TODO: Fields from configFile
    Creator creator = new Creator();

    creator.setCreatorName(OAIPMHUtilities.fieldLoader(this.jsonObj, "edition.agent.value"));
    NameIdentifier nameIdentifier = new NameIdentifier();
    // TODO: creatorID Field from configfile
    String creatorID =
        OAIPMHUtilities.fieldLoader(this.jsonObj, "edition.source.bibliographicCitation.author.id");

    if (creatorID != null && creatorID.contains(":")) {
      nameIdentifier.setValue(creatorID.split(":")[1]);
      nameIdentifier.setNameIdentifierScheme(creatorID.split(":")[0]);
      /*
       * TODO Check if the scheme URI is correct
       */
      nameIdentifier.setSchemeURI("https://ref.de.dariah.eu/pndsearch/pndquery.xql?id=");
    } else {
      nameIdentifier.setValue(creatorID);
    }
    creator.setNameIdentifier(nameIdentifier);
    nameIdentifier.setSchemeURI("https://ref.de.dariah.eu/pndsearch/pndquery.xql?id=");
    creators.getCreator().add(creator);

    return creators;

  }

  /**
   * @return
   */
  private Titles addTitles() {
    Titles titles = new Titles();
    for (String titleField : this.titleFields) {

      Title tileInOpenAireRecord = new Title();
      tileInOpenAireRecord.setValue(OAIPMHUtilities.fieldLoader(this.jsonObj, titleField));
      tileInOpenAireRecord.setTitleType(TitleType.OTHER);
      titles.getTitle().add(tileInOpenAireRecord);
    }

    return titles;
  }

  /**
   * @return
   */
  private static String addPublisher() {
    /**
     * Publisher fields
     */
    // TODO: fields from config file
    // TODO: for each field
    // return OAIPMHUtilities.fieldLoader(jsonObj,
    // "edition.source.bibliographicCitation.publisher.value");
    return "TextGrid";

  }

  /**
   * @return
   * @throws ParseException
   * @throws DatatypeConfigurationException
   */
  private String addPublicationYear() throws ParseException, DatatypeConfigurationException {
    /**
     * Publication Year fields
     */
    // TODO: fields from config file

    return Integer.toString(OAIPMHUtilities
        .convertDateFormat(OAIPMHUtilities.fieldLoader(this.jsonObj, "created")).getYear());
  }

  /**
   * @param formatField
   */
  public void setFormatField(String formatField) {
    this.formatField = formatField;
  }

  /**
   * @return
   */
  public String getFormatField() {
    return this.formatField;
  }

  /**
   * @param fields
   */
  public void setFields(String[] fields) {
    this.fields = fields;
  }

  /**
   * @return
   */
  public String[] getFields() {
    return this.fields;
  }

  public OAI_ESClient getOaiEsClient() {
    return this.oaiEsClient;
  }

  public void setOaiEsClient(OAI_ESClient oaiEsClient) {
    this.oaiEsClient = oaiEsClient;
  }

  public String[] getTitleFields() {
    return this.titleFields;
  }

  public void setTitleFields(String[] titleFields) {
    this.titleFields = titleFields;
  }

  public Resource getResource() {
    return this.resource;
  }

  public void setResource(Resource resource) {
    this.resource = resource;
  }

  public String[] getDateFields() {
    return this.dateFields;
  }

  public void setDateFields(String[] dateFields) {
    this.dateFields = dateFields;
  }

  public String[] getContributorFields() {
    return this.contributorFields;
  }

  public void setContributorFields(String[] contributorFields) {
    this.contributorFields = contributorFields;
  }

  public String[] getLanguageFields() {
    return this.languageFields;
  }

  public void setLanguageFields(String[] languageFields) {
    this.languageFields = languageFields;
  }

  public String[] getAlternateIdentifierFields() {
    return this.alternateIdentifierFields;
  }

  public void setAlternateIdentifierFields(String[] alternateIdentifierFields) {
    this.alternateIdentifierFields = alternateIdentifierFields;
  }

  public String[] getFormatFields() {
    return this.formatFields;
  }

  public void setFormatFields(String[] formatFields) {
    this.formatFields = formatFields;
  }



  public String[] getRightsFields() {
    return this.rightsFields;
  }

  public void setRightsFields(String[] rightsFields) {
    this.rightsFields = rightsFields;
  }

  public String[] getDescriptionFields() {
    return this.descriptionFields;
  }

  public void setDescriptionFields(String[] descriptionFields) {
    this.descriptionFields = descriptionFields;
  }

  public String[] getRelatedIdentifierFields() {
    return this.relatedIdentifierFields;
  }

  public void setRelatedIdentifierFields(String[] relatedIdentifierFields) {
    this.relatedIdentifierFields = relatedIdentifierFields;
  }

  public String getRelationToWorkObject() {
    return this.relationToWorkObject;
  }

  public void setRelationToWorkObject(String relationToWorkObject) {
    this.relationToWorkObject = relationToWorkObject;
  }

  /**
   * @return
   */
  public String[] getWorkFields() {
    return this.workFields;
  }

  /**
   * @param workFields
   */
  public void setWorkFields(String[] workFields) {
    this.workFields = workFields;
  }

  /**
   * @return
   */
  public String getIdentifierField() {
    return this.identifierField;
  }

  /**
   * @param identifierField
   */
  public void setIdentifierField(String identifierField) {
    this.identifierField = identifierField;
  }

  /**
   * @return
   */
  public String[] getGeoLocationFields() {
    return this.geoLocationFields;
  }

  /**
   * @param geoLocationFields
   */
  public void setGeoLocationFields(String[] geoLocationFields) {
    this.geoLocationFields = geoLocationFields;
  }

  /**
   * @return
   */
  public String getHandle() {
    return this.handle;
  }

  /**
   * @param handle
   */
  public void setHandle(String handle) {
    this.handle = handle;
  }

  /**
   * @return
   */
  public String[] getVersionFields() {
    return this.versionFields;
  }

  /**
   * @param versionFields
   */
  public void setVersionFields(String[] versionFields) {
    this.versionFields = versionFields;
  }

  /**
   * @return
   */
  public String[] getSubjectFields() {
    return this.subjectFields;
  }

  /**
   * @param subjectFields
   */
  public void setSubjectFields(String[] subjectFields) {
    this.subjectFields = subjectFields;
  }

  /**
   * @return
   */
  public String getSizeField() {
    return this.sizeField;
  }

  /**
   * @param size
   */
  public void setSizeField(String size) {
    this.sizeField = size;
  }

}
